/*
ADD MORE TESTS!
see t-printf.c?
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-logentry-0.c,v 1.29 2014/01/15 04:26:20 ca Exp $")
#include "sm/io.h"
#include "sm/error.h"
#include "sm/str.h"
#include "sm/cstr.h"
#include "sm/log.h"
#include "sm/net.h"
#include "sm/misc.h"
#include "sm/test.h"

#include <fcntl.h>

sm_logcategory_T c2_lcats[] = {
	{ "init", 	0 },
	{ "config", 	0 },
	{ "comm",	0 },
	{ "sched",	0 },
	{ NULL, 	0 }
};

sm_logmodule_T c2_lmods[] = {
	{ "comm",	 	0 },
	{ "config",		0 },
	{ "sched",		0 },
	{ NULL, 		0 }
};

#define C2_LCAT_INIT		(&c2_lcats[0])
#define C2_LCAT_COMM		(&c2_lcats[1])
#define C2_LCAT_SCHED		(&c2_lcats[2])

/* Backwards compatibility. */
#define C2_LCAT_GENERAL	QMGLOGCATEGORY_GENERAL

#define C2_LMOD_COMM		(&c2_lmods[0])
#define C2_LMOD_CONFIG		(&c2_lmods[1])
#define C2_LMOD_SCHED		(&c2_lmods[2])

sm_log_ctx_P	 c2_lctx;
sm_logconfig_P	 c2_lcfg;
int c2_loglevel = 9;

static sm_ret_T
test_init(void)
{
	sm_ret_T ret;

	c2_lctx = NULL;
	c2_lcfg = NULL;
	ret = sm_log_create(NULL, &c2_lctx, &c2_lcfg);
	SM_TEST_E(sm_is_success(ret));
	ret = sm_log_setfp_fd(c2_lctx, smioout, SMIOOUT_FILENO);
	SM_TEST_E(sm_is_success(ret));
	ret = sm_log_setdebuglevel(c2_lctx, c2_loglevel);
	SM_TEST_E(sm_is_success(ret));

  error:
	return ret;
}

#define MAXL	1024
#define STRL	64

static sm_ret_T
test_log(bool micro)
{
	sm_ret_T ret;
	int i;
	uint ui;
	sm_str_P str, strchk;
	sm_cstr_P cstr;
	char *cs;
	void *ptr;
	char buf[MAXL];
	sm_ipaddr_T ipaddr;
#if MTA_NETINET6
	ipv6_T ipv6;
	sm_sockaddr_T sockaddr;
#endif

	ret = SM_SUCCESS;
	str = sm_str_new(NULL, STRL, MAXL);
	SM_TEST_E(str != NULL);
	strchk = sm_str_new(NULL, STRL, MAXL);
	SM_TEST_E(strchk != NULL);

	if (micro) {
		ret = sm_log_setflags(c2_lctx, LOG_FL_MICRO);
		SM_TEST(SM_SUCCESS == ret);
	}

#if SM_LOG_TEST
	ret = sm_letest();
	if (SM_SUCCESS != ret)
		sm_io_fprintf(smioerr, "sm_letest=%#x\n", ret);
	SM_TEST_E(SM_SUCCESS == ret);
#endif

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_FUNC, "function",
		SLT_WHERE, "here",
		SLT_FREE, "name", "value",
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_FREE, "name1", "value1",
		SLT_FREE, "name2", "value2",
		SLT_FREE, "name3", "value3",
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	sm_str_clr(str);
	cs = "<a\t=c=,b@example.tld>";
	i = sm_str_scat(str, (const char *)cs);
	SM_TEST_E(i == SM_SUCCESS);

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_TA_ID, "S000000000010830600",
		SLT_MSGID, str,
		SLT_STATUS, "Sent",
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_TA_ID, "S001000000010830600",
		SLT_STAT, 250,
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_STAT, 471,
		SLT_STATUS|SLT_COND, (int) false, "Sent",
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	ptr = (void *)0xdeAc137f;
	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_SE_ID, "S00A000010010830600",
		SLT(SLT_STAT, 'x')|SLT_HASH, 0x1f,
		SLT(SLT_STAT, 'Z'), (ulong) 0x137f,
		SLT(SLT_STAT, 'Y'), (ulong) 0xf84Adeaf,
		SLT(SLT_FREE, 'p'), "ptr", ptr,
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	ptr = (void *)0xf071e835;
	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_SE_ID, "S11A111111111830610",
		SLT(SLT_STAT, 'o'), 0x1f,
		SLT(SLT_STAT, 'O'), 0x137f,
		SLT(SLT_STAT, 'X'), (ulong) 0xf84Adeaf,
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	sm_str_clr(str);
	cs = "1234567890abcdefghijklmnopqrstuvwxyz";
	i = sm_str_scat(str, (const char *)cs);
	SM_TEST_E(i == SM_SUCCESS);

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_SE_ID, "S00A0B0010010830F00",
		SLT(SLT_STATUS, 'S'), str,
		SLT(SLT_FREE, 'S')|SLT_PREC, 6, "prec", str,
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	cs = "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
	cstr = sm_cstr_crt((uchar *)cs, strlen(cs));
	SM_TEST_E(cstr != NULL);

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_SE_ID, "S00A0B00CD0E0830F01",
		SLT(SLT_FREE, 'C'), "cstr", cstr,
		SLT_FREE|SLT_WIDTH, 6, "width", "6",
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	sm_str_clr(str);
	cs = "ab%cd";
	i = sm_str_scat(str, (const char *)cs);
	SM_TEST_E(i == SM_SUCCESS);

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_DA_SE, "CF0A0B00CD0E0830F01",
		SLT(SLT_FREE, 'S')|SLT_PREC|SLT_CLEAN, 5, "string", str,
		SLT_NRCPTS, (uint)5,
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	buf[0] = 0x1a;
	buf[1] = 0x8f;
	buf[2] = 0xe1;
	buf[3] = 0xd7;
	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT(SLT_FREE, 's')|SLT_XTEXT, "xtext", "a b+c,d\te",
		SLT(SLT_FREE, 'y')|SLT_PREC|SLT_C, 4, "fp8", buf,
		SLT(SLT_FREE, 'y')|SLT_PREC|SLT_S, 2, "fp16", buf,
		SLT_END);

	buf[0] = 0xa;
	buf[1] = 0x8f;
	buf[2] = 0xe;
	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT(SLT_FREE, 'y')|SLT_ZEROPAD|SLT_PREC|SLT_C, 3, "fp8", buf,
		SLT_END);

	sm_str_clr(str);
	sm_strprintf(str, "%d\r\n", 0);
	i = sm_snprintf(buf, sizeof(buf), "%#T", str);
	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT(SLT_FREE, 'T')|SLT_HASH, "Thash", str,
		SLT(SLT_STAT, 'r'), (uint32_t) 0x80000001,
		SLT_IPV4, ntohl(0x7F000001),
		SLT_END);

	ret = sm_log_setdont(c2_lctx, LTD_FUNC|LTD_WHERE);
	SM_TEST(sm_is_success(ret));
	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_FUNC, "function",
		SLT_WHERE, "here",
		SLT_FREE, "newname", "newvalue",
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	ret = sm_log_setdont(c2_lctx, LTD_NONE);
	SM_TEST(sm_is_success(ret));
	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_FUNC, "function1",
		SLT_WHERE, "here1",
		SLT_FREE, "name", "value1",
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT(SLT_FREE, 'x')|SLT_L, "long", (long) 0x87654321,
		SLT(SLT_FREE, 'x')|SLT_S, "short", (short) 0x8FAD,
		SLT(SLT_FREE, 'x')|SLT_C, "char", (char) 0x7E,
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

#define T1ADDR "12.234.99.39"
	i = sm_inet_a2ip(T1ADDR, NULL, &ipaddr);
	SM_TEST(SM_SUCCESS == i);
	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT(SLT_IP, 'I'), &ipaddr,
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	ret = sm_error_perm(SM_EM_IO, EEXIST);
	i = 0;
	cs = NULL;
	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLE_EM("io", ret),
		SLT(SLT_FREE, 'm')|SLT_COND, sm_is_err(ret),  "IBDB", ret,
		SLT_MOD|SLT_COND, sm_is_err(ret), ret,
		SLT(SLT_FREE, 'm')|SLT_COND, sm_is_err(i),  "NONE", i,
		SLT_MOD|SLT_COND, sm_is_err(i), i,
		SLT(SLT_FREE, 'm')|SLT_COND, sm_is_err(i),  "NONE", i,
		SLT(SLT_FREE, 's')|SLT_COND, (cs != NULL), cs, cs,
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	ret = sm_error_perm(SM_EM_IO, EEXIST);
	i = 421;
	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_STATM, ret,
		SLT_STATM, i,
		SLT_END);
	SM_TEST_E(sm_is_success(ret));

	/* need to skip past timestamp */
#define SKIP_TMSTAMP	do { \
	ret = sm_str_f1chr(str, (uchar)' ', 0, &ui);	\
	SM_TEST_E(sm_is_success(ret));	\
	SM_TEST_E(ui < sm_str_getlen(str));	\
	} while (0)

	/* log to a str */
	sm_str_clr(str);
	sm_str_clr(strchk);

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_SET_FP_FIRST, str,
		SLT_TA_ID, "S000000000010830600",
		SLT_STATUS, "Sent",
		SLT_END);
	SM_TEST_E(sm_is_success(ret));
	ret = sm_str_scat(strchk, " ta_id=S000000000010830600, status=Sent");
	SM_TEST_E(sm_is_success(ret));
	SKIP_TMSTAMP;
	SM_TEST(sm_str_casecmp_off(str, ui, strchk, 0) == 0);

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_SET_FP_LAST, str,
		SLT_FREE, "str", "flushed",
		SLT_END);
	SM_TEST_E(sm_is_success(ret));
	ret = sm_str_scat(strchk, ", str=flushed");
	SM_TEST_E(sm_is_success(ret));
	SKIP_TMSTAMP;
	SM_TEST(sm_str_casecmp_off(str, ui, strchk, 0) == 0);

	sm_str_clr(str);
	sm_str_clr(strchk);

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_SET_FP_FIRST, str,
		SLT_TA_ID, "S001000000010830600",
		SLT_END);
	SM_TEST_E(sm_is_success(ret));
	ret = sm_str_scat(strchk, " ta_id=S001000000010830600");
	SM_TEST_E(sm_is_success(ret));
	SKIP_TMSTAMP;
	SM_TEST(sm_str_casecmp_off(str, ui, strchk, 0) == 0);

	for (i = 0; i < 8; i++) {
		ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
			SLT_SET_FP_MORE, str,
			SLT_IDX_D, i,
			SLT_END);
		SM_TEST_E(sm_is_success(ret));
		ret = sm_strprintf(strchk, ", idx=%d", i);
		SM_TEST_E(sm_is_success(ret));
		SKIP_TMSTAMP;
		SM_TEST(sm_str_casecmp_off(str, ui, strchk, 0) == 0);
	}

	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT_SET_FP_LAST, str,
		SLT_FREE, "string", "Flushed",
		SLT_END);
	SM_TEST_E(sm_is_success(ret));
	ret = sm_str_scat(strchk, ", string=Flushed");
	SM_TEST_E(sm_is_success(ret));
	SKIP_TMSTAMP;
	SM_TEST(sm_str_casecmp_off(str, ui, strchk, 0) == 0);


#if MTA_NETINET6
#define TADDR "fe80::2a0:c9ff:fee6:826b"
	i = sm_inet_a2ipv6(TADDR, NULL, &ipv6);
	SM_TEST(SM_SUCCESS == i);
	ipaddr.ipa_family = AF_INET6;
	IPV6_ASSIGN(&ipaddr.ipa_addr.ipv6_addr, &ipv6);
	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT(SLT_IPV6, 'I'), &ipaddr,
		SLT_END);
	SM_TEST_E(sm_is_success(ret));
#endif

#if MTA_NETINET6
	i = sm_inet_a2ipv6("::1", NULL, &ipv6);
	SM_TEST(SM_SUCCESS == i);
	sockaddr.sin6.sin6_family = AF_INET6;
	IPV6_ASSIGN(&sockaddr.sin6.sin6_addr, &ipv6);
	ret = sm_log_entry(c2_lctx, C2_LCAT_INIT, C2_LMOD_CONFIG, SM_LOG_INFO, 8,
		SLT(SLT_IPV6, 'a'), &sockaddr,
		SLT_END);
	SM_TEST_E(sm_is_success(ret));
#endif

  error:
	return ret;
}


int
main(int argc, char *argv[])
{
	int c;
	sm_ret_T ret;
	bool micro;

	micro = false;
	while ((c = getopt(argc, argv, "?bC")) != -1) {
		switch (c) {
		  case 'b':
			micro = true;
			break;
		  case 'C':
			compoptions(smioout);
			exit(0);
#if 0
		  default:
			usage(argv[0]);
			return(1);
#endif /* 0 */
		}
	}

	sm_test_begin(argc, argv, "test logentry 0");

	ret = test_init();
	SM_TEST_E(sm_is_success(ret));

	ret = test_log(micro);
	SM_TEST_E(sm_is_success(ret));

	sm_log_destroy(c2_lctx);
  error:
	return sm_test_end();
}
