/*
 * Copyright (c) 2003-2006 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: log.h,v 1.62 2014/03/14 17:43:53 ca Exp $
 */

#ifndef SM_LOG_H
#define SM_LOG_H 1

#include "sm/generic.h"
#include "sm/varargs.h"
#include "sm/io.h"
#include "sm/syslog.h"

#if !HAVE_VSYSLOG
void	vsyslog(int _level, const char *_format, va_list _args);
#endif

/* see io.h */
#if 1
#define SMIOLOG_FILENO		SMIOERR_FILENO
#else
#define SMIOLOG_FILENO		SMIOOUT_FILENO
#endif
#define smiolog		(&SmIoF[SMIOLOG_FILENO])

/* Hack ... */
#define SM_LOG_EMERG	LOG_EMERG
#define SM_LOG_FATAL	LOG_EMERG
#define SM_LOG_ALERT	LOG_ALERT
#define SM_LOG_INCONS	LOG_CRIT	/* internal inconsistency */
#define SM_LOG_CRIT		LOG_CRIT
#define SM_LOG_ERROR	LOG_ERR
#define SM_LOG_ERR		SM_LOG_ERROR
#define SM_LOG_WARNING	LOG_WARNING
#define SM_LOG_WARN		SM_LOG_WARNING
#define SM_LOG_NOTICE	LOG_NOTICE
#define SM_LOG_INFO		LOG_INFO
#define SM_LOG_DEBUG	LOG_DEBUG
#define SM_LOG_DBG		LOG_DEBUG

#define SLT_SET_FP_FIRST	(-1)	/* set fp for logging */
#define SLT_SET_FP_MORE		(-2)	/* set fp for logging; more data */
#define SLT_SET_FP_LAST		(-3)	/* set fp for logging; last call */
#define SLT_IS_SET_FP(slt)	(SLT_SET_FP_FIRST == (slt)||\
							 SLT_SET_FP_MORE == (slt) ||\
							 SLT_SET_FP_LAST == (slt))

/* new logging API */
/* !}./bin/lt2def */
#define SLT_END               0
#define SLT_FREE              1
#define SLT_SEV               2
#define SLT_FUNC              3
#define SLT_WHERE             4
#define SLT_SS_SE             5
#define SLT_SS_TA             6
#define SLT_MAIL              7
#define SLT_SE_ID             8
#define SLT_SS_ID             9
#define SLT_TA_ID            10
#define SLT_TA               11
#define SLT_RCPT             12
#define SLT_RCPT_PA          13
#define SLT_RCPT_ID          14
#define SLT_RCPT_IDX         15
#define SLT_RCPT_B           16
#define SLT_RCPT_B_IDX       17
#define SLT_RCPT_IDX_DSN     18
#define SLT_NRCPTS           19
#define SLT_I                20
#define SLT_CDB              21
#define SLT_MSGID            22
#define SLT_SIZE             23
#define SLT_STAT             24
#define SLT_STAT_M           25
#define SLT_STATE            26
#define SLT_STATUS           27
#define SLT_STATUS_T         28
#define SLT_STATM            29
#define SLT_ERROR            30
#define SLT_INPUT            31
#define SLT_MOD              32
#define SLT_DA_SE            33
#define SLT_DA_TA            34
#define SLT_REUSE            35
#define SLT_KEEP_OPEN        36
#define SLT_E_CUR            37
#define SLT_E_MAX            38
#define SLT_OPN_CONN         39
#define SLT_CUR_LIMIT        40
#define SLT_CUR_RATE         41
#define SLT_RATE_MAX         42
#define SLT_TIMEOUT          43
#define SLT_CERT_ISS         44
#define SLT_CERT_SUB         45
#define SLT_CN               46
#define SLT_CIPHER           47
#define SLT_USE_BITS         48
#define SLT_ALG_BITS         49
#define SLT_TLS_REUSED       50
#define SLT_VERIFY           51
#define SLT_CLT              52
#define SLT_CLT_IP           53
#define SLT_CLT_NAME         54
#define SLT_SRV              55
#define SLT_REPLY            56
#define SLT_IP               57
#define SLT_IPV4             58
#define SLT_IPV6             59
#define SLT_PORT             60
#define SLT_LOCK             61
#define SLT_NTRIES           62
#define SLT_MT_RHS           63
#define SLT_SC_THR           64
#define SLT_PID              65
#define SLT_SS_MCT           66
#define SLT_SS_AVLB          67
#define SLT_IDX_U            68
#define SLT_IDX_D            69
#define SLT_RET              70
#define SLT_DELAY            71
#define SLT_XDELAY           72
#define SLT_CDELAY           73
#define SLT_EDELAY           74
#define SLT_TDELAY           75
#define SLT_MDELAY           76
#define SLT_RDELAY           77
#define SLT_DDELAY           78
#define SLT_BDELAY           79
#define SLT_FDELAY           80
#define SLT_NOW_ENTERED      81
#define SLT_IID              82
#define SLT_PRI              83
#define SLT_DKIMS            84
#define SLT_LAST             85

#define BOOL2PRT(b)	((b) ? "yes" : "no")

/*
 *  Log error code and module: SLE_EM("function", ret) ->
 *  function=%m, module=%H
 */
#define SLE_EM(name, ret)	SLT(SLT_FREE, 'm'), name, (ret), SLT_MOD, (ret)

/*
 *  layout:
 *	bit 31-26	currently unused
 *	bit 25		condition?
 *	bit 24		@?
 *	bit 23		#?
 *	bit 22		precision?
 *	bit 21		width?
 *	bit 20-18	modifier (3 bits: there are 7 modifiers, 0: none)
 *			hh, h, l, ll, j, z, t; the last 3 are not yet implemented
 *	bit 17-12	type (6 bits: 0-63), basically format specifier
 *	bit 11- 0	field (12 bits, 0-4095)
 */

#define SM_LOG_FIELD_BITS	0x000007ff
#define SM_LOG_TYPE_BITS	0x0000003f
#define SM_LOG_MOD_BITS		0x00000007
#define SM_LOG_TYPE_OFF		11
#define SM_LOG_MOD_OFF		17

/* modifiers */
#define SLT_LV		1
#define SLT_LLV		2
#define SLT_SV		3
#define SLT_CV		4
#define SLT_L		(SLT_LV << SM_LOG_MOD_OFF)		/* long integer */
#define SLT_LL		(SLT_LLV << SM_LOG_MOD_OFF)		/* long long integer */
#define SLT_S		(SLT_SV << SM_LOG_MOD_OFF)		/* short integer */
#define SLT_C		(SLT_CV << SM_LOG_MOD_OFF)		/* 8 bit integer */

#define SLT_WIDTH	0x00200000
#define SLT_PREC	0x00400000
#define SLT_HASH	0x00800000
#define SLT_CLEAN	0x01000000
#define SLT_COND	0x02000000
#define SLT_XTEXT	0x04000000
#define SLT_ZEROPAD	0x08000000
 
#define SM_LOGT_OFF	'@'

#define sm_log_field(lt)	((lt) & SM_LOG_FIELD_BITS)
#define sm_log_type(lt)	(((lt) >> SM_LOG_TYPE_OFF) & SM_LOG_TYPE_BITS)
#define sm_log_t2f(lt)	(sm_log_type(lt) + SM_LOGT_OFF)
#define sm_log_f2t(f)	((f) - SM_LOGT_OFF)
#define sm_log_mod(lt)	(((lt) >> SM_LOG_MOD_OFF) & SM_LOG_MOD_BITS)
#define sm_log_ishash(lt)	((lt) & SLT_HASH)
#define sm_log_isclean(lt)	((lt) & SLT_CLEAN)
#define sm_log_isxtext(lt)	((lt) & SLT_XTEXT)
#define sm_log_iscond(lt)	((lt) & SLT_COND)
#define sm_log_isprec(lt)	((lt) & SLT_PREC)
#define sm_log_iswidth(lt)	((lt) & SLT_WIDTH)
#define sm_log_is0pad(lt)	((lt) & SLT_ZEROPAD)

#define SLT(field, type)	(((field) & SM_LOG_FIELD_BITS)|\
	 ((uint32_t)(sm_log_f2t(type) & SM_LOG_TYPE_BITS) << SM_LOG_TYPE_OFF))

#define LTD_NONE	0
#define LTD_FUNC	0x00000001
#define LTD_WHERE	0x00000002

/* end new logging API */

#define LOG_FL_MICRO	0x0001

typedef struct sm_log_ctx_S 		sm_log_ctx_T, *sm_log_ctx_P;
typedef struct sm_logcategory_S		sm_logcategory_T, *sm_logcategory_P;
typedef struct sm_logconfig_S		sm_logconfig_T, *sm_logconfig_P;
typedef struct sm_logmodule_S		sm_logmodule_T, *sm_logmodule_P;
typedef struct sm_logdestination_S	sm_logdestination_T, *sm_logdestination_P;
typedef struct sm_rpool_S		sm_mem_T, *sm_mem_P;

/*
**  Used to name the categories used by a library.  An array of sm_logcategory
**  structures names each category.
*/

struct sm_logcategory_S {
	const char	*smlc_name;
	uint		 smlc_log_level;
};

/*
** Similar to sm_logcategory above, but for all the modules a library defines.
*/

struct sm_logmodule_S {
	const char	*smlm_name;
	uint		 smlm_log_level;
};


/*
**  As a first hack just map this stuff to some syslog functions?
*/

sm_ret_T sm_log_create(sm_mem_P _mctx, sm_log_ctx_P *_plctx, sm_logconfig_P *_plcfg);
sm_ret_T sm_log_destroy(sm_log_ctx_P _lctx);

sm_ret_T sm_logconfig_create(sm_log_ctx_P _lctx, sm_logconfig_P *_plcfg);
sm_ret_T sm_logconfig_use(sm_log_ctx_P _lctx, sm_logconfig_P _lcfg);
sm_logconfig_P	 sm_logconfig_get(sm_log_ctx_P _lctx);
sm_ret_T sm_logconfig_destroy(sm_logconfig_P _lcfg);

sm_ret_T sm_log_registercategories(sm_log_ctx_P _lctx, sm_logcategory_T categories[]);
sm_ret_T sm_log_registermodules(sm_log_ctx_P _lctx, sm_logmodule_T modules[]);

sm_ret_T sm_log_createchannel(sm_logconfig_P _lcfg, const char *_name,
		      uint _type, int _priority, uint _level,
		      const sm_logdestination_P _destination,
		      uint _flags);
sm_ret_T sm_log_usechannel(sm_logconfig_P _lcfg, const char *_name,
		   const sm_logcategory_P _category,
		   const sm_logmodule_P _module);

sm_ret_T sm_log_setfile(sm_log_ctx_P _lctx, sm_file_T *_file);
sm_ret_T sm_log_setfilename(sm_log_ctx_P _lctx, char *_name);

sm_ret_T SM_PRINTFLIKE(6, 7)
sm_log_write(sm_log_ctx_P _lctx, sm_logcategory_P _category,
			sm_logmodule_P _module, int _priority, uint _level, const char *_format, ...);
sm_ret_T sm_log_vwrite(sm_log_ctx_P _lctx, sm_logcategory_P _category,
			sm_logmodule_P _module, int _priority, uint _level,
			const char *_format, va_list _args);
#if 0
sm_ret_T SM_PRINTFLIKE(6, 7)
sm_log_write1(sm_log_ctx_P _lctx, sm_logcategory_P _category,
	       sm_logmodule_P _module, int _priority, uint _level, const char *_format, ...);
sm_ret_T sm_log_vwrite1(sm_log_ctx_P _lctx, sm_logcategory_P _category,
		sm_logmodule_P _module, int _priority, uint _level,
		const char *_format, va_list _args);

sm_ret_T sm_log_iwrite(sm_log_ctx_P _lctx, sm_logcategory_P _category,
	       sm_logmodule_P _module, int _priority, uint _level,
	       sm_msgcat_P msgcat, int _msgset, int _msg,
	       const char *_format, ...);
sm_ret_T sm_log_ivwrite(sm_log_ctx_P _lctx, sm_logcategory_P _category,
	       sm_logmodule_P _module, int _priority, uint _level,
	       sm_msgcat_P msgcat, int _msgset, int _msg,
	       const char *_format, va_list _args);
sm_ret_T sm_log_iwrite1(sm_log_ctx_P _lctx, sm_logcategory_P _category,
		sm_logmodule_P _module, int _priority, uint _level,
		sm_msgcat_P msgcat, int _msgset, int _msg,
		const char *_format, ...);
sm_ret_T sm_log_ivwrite1(sm_l_ctxog_P _lctx, sm_logcategory_P _category,
		 sm_logmodule_P _module, int _priority, uint _level,
		 sm_msgcat_P msgcat, int _msgset, int _msg,
		 const char *_format, va_list _args);
#endif /* 0 */

sm_ret_T sm_log_setdebuglevel(sm_log_ctx_P _lctx, uint _level);
uint	 sm_log_getdebuglevel(sm_log_ctx_P _lctx);
bool	 sm_log_wouldlog(sm_log_ctx_P _lctx, sm_logcategory_P _category, sm_logmodule_P _module, uint _level);
#if 0
sm_ret_T sm_log_setduplicateinterval(sm_logconfig_P _lcfg, uint _interval);
uint	 sm_log_getduplicateinterval(sm_logconfig_P _lcfg);
#endif /* 0 */
sm_ret_T sm_log_settag(sm_logconfig_P _lcfg, const char *_tag);
sm_ret_T sm_log_gettag(sm_logconfig_P _lcfg);
sm_ret_T sm_log_opensyslog(const char *_tag, int _options, int _facility);
sm_ret_T sm_log_closesyslog(void);
sm_ret_T sm_log_closefilelogs(sm_log_ctx_P _lctx);
sm_ret_T sm_log_categorybyname(sm_log_ctx_P _lctx, const char *_name, sm_logcategory_P *_pcategories);
sm_ret_T sm_log_modulebyname(sm_log_ctx_P _lctx, const char *_name, sm_logmodule_P *_pmodules);
sm_ret_T sm_log_setcontext(sm_log_ctx_P _lctx);
sm_ret_T sm_log_setfp_fd(sm_log_ctx_P _lctx, sm_file_T *_fp, int _fd);
sm_ret_T sm_log_setfd(sm_log_ctx_P _lctx, int _fd);
sm_ret_T sm_log_setflags(sm_log_ctx_P _lctx, uint32_t _flags);

sm_ret_T sm_log_reopen(sm_log_ctx_P _lctx);

sm_ret_T sm_log_cat_set_level(sm_log_ctx_P _lctx, const char *_name, uint _level);
sm_ret_T sm_log_mod_set_level(sm_log_ctx_P _lctx, const char *_name, uint _level);
char	*sm_log_sev2txt(int _loglevel);

sm_ret_T sm_log_entry(sm_log_ctx_P _lctx, sm_logcategory_P _category, sm_logmodule_P _module, int _priority, uint _level, ...);

sm_ret_T sm_log_setdont(sm_log_ctx_P _lctx, uint32_t _dont);
uint32_t sm_log_getdont(sm_log_ctx_P _lctx);

const char *sm_log_utstamp(void);
const char *sm_log_tstamp(bool _micro);

#endif /* SM_LOG_H */
