/*
 *  code to evaluate an expression "tree".
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: sm-expr-eval.c,v 1.16 2011/01/30 21:48:53 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/io.h"
#include <regex.h>	/* sm/ wrapper? */
#include "sm/sm-conf.h"
#include "sm/sm-conf-expr.h"
#include "sm-expr.h"
#include "cond.h"

#include <stdio.h>	/* debugging */

sm_func_T functab[] = {
	 { ID_STR("match"), F_MATCH }
	,{ ID_STR("strcasecmp"), F_STRCASECMP }
	,{ ID_STR("strcmp"), F_STRCMP }
};

SMBSRCHN(e, sm_id_entry_P, ide_id, ide_len)

static sm_ret_T
getstrval(cexpr_P expr, uint exni, sm_eval_ctx_P evalctx, const char **str, uint *str_len)
{
	int idx;

	SM_REQUIRE(expr != NULL);
	SM_REQUIRE(str != NULL);
	SM_REQUIRE(str_len != NULL);

	if (CXN(expr, exni).cen_vt == vt_str) {
		*str = CXN(expr, exni).cen_eval.cv_id.id_str;
		*str_len = CXN(expr, exni).cen_eval.cv_id.id_len;
		return SM_SUCCESS;
	}

	if (CXN(expr, exni).cen_vt == vt_id && evalctx->ec_idgetval != NULL &&
		(idx = CXN(expr, exni).cen_val.cv_id.id_idx) >= 0)
	{
		val_type_T vt;
		int err;

		vt = vt_str;
		err = evalctx->ec_idgetval((sm_conf_T *)NULL,
			evalctx->ec_idgetvalctx, idx,
			(sm_conf_definition_T const *) NULL,
			&vt, str, str_len);
		return err;
	}

	*str = NULL;
	*str_len = 0;
/*
printf("eval: ERROR: getstrval=-1\n");
*/
	return sm_err_perm(EINVAL);
}

static sm_ret_T
getintval(cexpr_P expr, uint exni, sm_eval_ctx_P evalctx, int *pval)
{
	int idx;

	SM_REQUIRE(expr != NULL);
	SM_REQUIRE(pval != NULL);

	if (/*CXN(expr, exni).cen_vt == vt_int ||*/ CXN(expr, exni).cen_evt == vt_int) {
		*pval = CXN(expr, exni).cen_eval.cv_int;
		return SM_SUCCESS;
	}

	if (CXN(expr, exni).cen_vt == vt_id && evalctx->ec_idgetval != NULL &&
		(idx = CXN(expr, exni).cen_val.cv_id.id_idx) >= 0)
	{
		val_type_T vt;
		int err;

		vt = vt_int;
		err = evalctx->ec_idgetval((sm_conf_T *)NULL,
			evalctx->ec_idgetvalctx, idx,
			(sm_conf_definition_T const *) NULL,
			&vt, pval, NULL);
		return err;
	}

	*pval = -1;
/*
printf("eval: ERROR: getintval=-1\n");
*/
	return sm_err_perm(EINVAL);
}

static int functabsz = SM_ARRAY_SIZE(functab);
SMBINSRCHN(f, functab, functabsz, f_id, f_len)

sm_ret_T
eval_exp(cexpr_P expr, uint exni, sm_eval_ctx_P evalctx)
{
	sm_ret_T ret;
	int a0, a1, r;

	if (NULL == expr)
		return sm_err_perm(EINVAL);

	if (CXN(expr, exni).cen_nargs == 0) {
		if (CXN(expr, exni).cen_vt == vt_int) {
			CXN(expr, exni).cen_evt = CXN(expr, exni).cen_vt;
			CXN(expr, exni).cen_eval.cv_int = CXN(expr, exni).cen_val.cv_int;
			return SM_SUCCESS;
		}
		else if (CXN(expr, exni).cen_vt == vt_id) {
			int i;

			i = e_bsrchn(evalctx->ec_idtab, evalctx->ec_idtabsz,
					CXN(expr, exni).cen_val.cv_id.id_str,
					CXN(expr, exni).cen_val.cv_id.id_len);
			if (i < 0)
				return sm_err_perm(EINVAL);

			if (CXN(expr, exni).cen_val.cv_id.id_vt != evalctx->ec_idtab[i].ide_vt) {
/*
printf("eval: id=\"%.*s\", id-type=%d, tab-type=%d\n",
CXN(expr, exni).cen_val.cv_id.id_len, CXN(expr, exni).cen_val.cv_id.id_str,
CXN(expr, exni).cen_val.cv_id.id_vt, evalctx->ec_idtab[i].ide_vt);
*/
				return sm_err_perm(EINVAL);
			}
/*
printf("eval: id=\"%.*s\", id-type=%d, flags=%#x\n",
CXN(expr, exni).cen_val.cv_id.id_len, CXN(expr, exni).cen_val.cv_id.id_str,
CXN(expr, exni).cen_val.cv_id.id_vt, evalctx->ec_idtab[i].ide_flags);
*/
			if (vt_int == evalctx->ec_idtab[i].ide_vt &&
				(evalctx->ec_idtab[i].ide_flags & IDEF_CONST_INT) != 0)
			{
				CXN(expr, exni).cen_evt = vt_int;
				CXN(expr, exni).cen_eval.cv_int = evalctx->ec_idtab[i].ide_val;
			}
#if 0
			else if (vt_bool == evalctx->ec_idtab[i].ide_vt &&
					(evalctx->ec_idtab[i].ide_flags & IDEF_CONST_BOOL) != 0)
			{
				CXN(expr, exni).cen_evt = vt_bool;
				CXN(expr, exni).cen_eval.cv_bool = evalctx->ec_idtab[i].ide_val;
			}
			else if (vt_str == evalctx->ec_idtab[i].ide_vt) {
				/* evaluate when needed */
			}
			else {
/*
printf("eval: id=\"%*s\", tab-type=%d\n",
CXN(expr, exni).cen_val.cv_id.id_len, CXN(expr, exni).cen_val.cv_id.id_str,
CXN(expr, exni).cen_val.cv_id.id_vt, evalctx->ec_idtab[i].ide_vt);
*/
				return sm_err_perm(EINVAL);
			}
#endif

			return SM_SUCCESS;
		}
		else if (CXN(expr, exni).cen_vt == vt_str) {
			CXN(expr, exni).cen_evt = CXN(expr, exni).cen_vt;
			CXN(expr, exni).cen_eval.cv_id = CXN(expr, exni).cen_val.cv_id;
			return SM_SUCCESS;
		}
		return sm_err_perm(EINVAL);
	}
	if (CXN(expr, exni).cen_nargs == 2) {
		bool seqeval, shortcut;

		seqeval = shortcut = false;
		switch (CXN(expr, exni).cen_val.cv_op) {
		  case OP_AND:
		  case OP_OR:
			seqeval = true;
			ret = eval_exp(expr, CXN(expr, exni).cen_args[0], evalctx);
			if (sm_is_err(ret)) return ret;

			/* should be bool... */
			a0 = CXN(expr, CXN(expr, exni).cen_args[0]).cen_eval.cv_int;
			if ((a0 && CXN(expr, exni).cen_val.cv_op == OP_OR) ||
			    (!a0 && CXN(expr, exni).cen_val.cv_op == OP_AND))
			{
				CXN(expr, exni).cen_evt = vt_int;
				CXN(expr, exni).cen_eval.cv_int = a0;
				return SM_SUCCESS;
			}
			break;
		}

		if (!seqeval) {
			ret = eval_exp(expr, CXN(expr, exni).cen_args[0], evalctx);
			if (sm_is_err(ret)) return ret;
		}
		ret = eval_exp(expr, CXN(expr, exni).cen_args[1], evalctx);
		if (sm_is_err(ret)) return ret;

		switch (CXN(expr, exni).cen_val.cv_op) {
		  case OP_AND:
		  case OP_OR:
		  case OP_PLUS:
		  case OP_MINUS:
		  case OP_MUL:
		  case OP_DIV:
		  case OP_MOD:
		  case OP_LT:
		  case OP_GT:
		  case OP_LE:
		  case OP_GE:
		  case OP_EQ:
		  case OP_NE:
			ret = getintval(expr, CXN(expr, exni).cen_args[0], evalctx, &a0);
			if (sm_is_err(ret)) return ret;
			ret = getintval(expr, CXN(expr, exni).cen_args[1], evalctx, &a1);
			if (sm_is_err(ret)) return ret;
			break;
		}

		switch (CXN(expr, exni).cen_val.cv_op) {
		  case OP_AND: r = a0 && a1;
			break;
		  case OP_OR: r = a0 || a1;
			break;
		  case OP_PLUS: r = a0 + a1;
			break;
		  case OP_MINUS: r = a0 - a1;
			break;
		  case OP_MUL: r = a0 * a1;
			break;
		  case OP_DIV: r = a0 / a1;
			break;
		  case OP_MOD: r = a0 % a1;
			break;
		  case OP_LT: r = a0 < a1;
			break;
		  case OP_GT: r = a0 > a1;
			break;
		  case OP_LE: r = a0 <= a1;
			break;
		  case OP_GE: r = a0 >= a1;
			break;
		  case OP_EQ: r = a0 == a1;
			break;
		  case OP_NE: r = a0 != a1;
			break;
		  case OP_MATCH:
		  case F_MATCH:
		  case F_STRCASECMP:
		  case F_STRCMP:
			{
				const char *str1, *str2;
				uint str1_len, str2_len, len;
				regex_t re;

				ret = getstrval(expr, CXN(expr, exni).cen_args[0], evalctx, &str1, &str1_len);
				if (sm_is_err(ret)) return ret;
				ret = getstrval(expr, CXN(expr, exni).cen_args[1], evalctx, &str2, &str2_len);
				if (sm_is_err(ret)) return ret;
				len = SM_MIN(str1_len, str2_len);

				switch (CXN(expr, exni).cen_val.cv_op) {
				  case OP_MATCH:
				  case F_MATCH:
					/* compile this only once at parsing time? */
					r = regcomp(&re, str2, REG_EXTENDED|REG_ICASE|REG_NOSUB);
					if (r != 0)
						return sm_err_perm(EINVAL); /* better error code! */
	    			r = regexec(&re, str1, 0, NULL, 0) == 0;
/*
printf("eval: str=\"%s\", pattern=\"%s\", r=%d\n", str1, str2, r);
*/
	    			(void) regfree(&re);
					break;

				  case F_STRCASECMP:
					r = strncasecmp(str1, str2, len);
					if (r == 0) r = str1_len - str2_len;
					break;
				  case F_STRCMP:
					r = strncmp(str1, str2, len);
/*
printf("eval: str1=\"%s\", str2=\"%s\", strcmp=%d\n", str1, str2, r);
*/
					if (r == 0) r = str1_len - str2_len;
					break;
				  default:
					SM_ASSERT(0);
					/* NOTREACHED */
					r = -1;
					break;
				}
/*
printf("eval: str1=\"%.*s\", str2=\"%.*s\", r=%d\n", str1_len, str1, str2_len, str2, r);
*/

			}
			break;
		  default:
			return sm_err_perm(EINVAL);
		}
		/* HACK */
		CXN(expr, exni).cen_evt = vt_int;
		CXN(expr, exni).cen_eval.cv_int = r;
		return SM_SUCCESS;
	}
	if (CXN(expr, exni).cen_nargs == 1) {

		ret = eval_exp(expr, CXN(expr, exni).cen_args[0], evalctx);
		if (sm_is_err(ret)) return ret;
		a0 = CXN(expr, CXN(expr, exni).cen_args[0]).cen_eval.cv_int;
		switch (CXN(expr, exni).cen_val.cv_op) {
		  case OP_NOT: r = !a0;
			break;
		  case UMINUS: r = -a0;
			break;
		  default:
			return sm_err_perm(EINVAL);
		}
		CXN(expr, exni).cen_evt = vt_int;
		CXN(expr, exni).cen_eval.cv_int = r;
		return SM_SUCCESS;
	}
/*
printf("eval: ERROR: nargs=%d\n", CXN(expr, exni).cen_nargs);
*/
	return -1;
}
