/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-conf-union-choice-1.c,v 1.3 2012/10/26 15:35:27 ca Exp $")

#include "sm/string.h"
#include "sm/net.h"
#include "sm/sm-conf.h"
#include "sm/sm-conf-prt.h"
#include <stdio.h>

/*
 *  Test using the same flag variable (struct entry) in different options.
 */

#define QCNF_FL_VERP_NEVER	0x00000100	/* never use VERP */
#define QCNF_FL_VERP_ALWAYS	0x00000200	/* always use VERP */
#define QCNF_FL_VERP_ADD	0x00000400	/* add another item to sender address */

typedef struct qmgr_cnf_S	qmgr_cnf_T, *qmgr_cnf_P;
struct qmgr_cnf_S
{
	uint32_t	 q_cnf_flags; /* configuration flags, see qmgr-int.h */
};

sm_conf_definition_T const
qmgr_verp_when_names[] = {
{ SM_CONF_DEF_MAGIC, "never",	sm_conf_type_choice_value,
	QCNF_FL_VERP_NEVER	,
	0, NULL, 0, NULL, NULL, NULL,
	"never use VERP"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "when_requested",	sm_conf_type_choice_value,
	0,
	0, NULL, 0, NULL, NULL, NULL,
	"use VERP when requested via XVERP or verp- in aliases"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "always",	sm_conf_type_choice_value,
	QCNF_FL_VERP_ALWAYS,
	0, NULL, 0, NULL, NULL, NULL,
	"always use VERP"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
};

sm_conf_definition_T const
qmgr_verp_add_names[] = {
{ SM_CONF_DEF_MAGIC, "nothing",	sm_conf_type_choice_value,
	0,
	0, NULL, 0, NULL, NULL, NULL,
	"add nothing extra"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "jobid",	sm_conf_type_choice_value,
	QCNF_FL_VERP_ADD,
	0, NULL, 0, NULL, NULL, NULL,
	"add jobid (not yet available)"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
};

sm_conf_definition_T
qmgr_verp_defs[] = {
{ SM_CONF_DEF_MAGIC, "when",		sm_conf_type_choice,
	offsetof(qmgr_cnf_T, q_cnf_flags),		sizeof(uint),
	NULL,	SM_CONF_FLAG_OR,
	qmgr_verp_when_names,	NULL,	NULL,
	"when to use VERP"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "add",		sm_conf_type_choice,
	offsetof(qmgr_cnf_T, q_cnf_flags),		sizeof(uint),
	NULL,	SM_CONF_FLAG_OR,
	qmgr_verp_add_names,	NULL,	NULL,
	"add something else to sender address"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
};

static sm_conf_definition_T mts_def[] =
{

{ SM_CONF_DEF_MAGIC, "VERP",	sm_conf_type_section,
	0, sizeof(qmgr_cnf_T), NULL, SM_CONF_FLAG_KEEP_DEFAULT, qmgr_verp_defs,
	NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
};

static qmgr_cnf_T mts;

static void
print_mts(qmgr_cnf_P mts)
{
}

static int
process(char const *name, FILE *fp)
{
	sm_conf_T *stream;
	int err;
	char buf[SM_CONF_ERROR_BUFFER_SIZE];
	char const *e = NULL;

	if (((stream = sm_conf_new(name ? name : "*stdin*"))) == NULL) {
		fprintf(stderr, "error -- sm_conf_new() returns NULL!\n");
		return 1;
	}
	if ((err = sm_conf_read_FILE(stream, name, fp)) != 0) {
		fprintf(stderr, "%s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));

		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			fprintf(stderr, "%s\n", e);

		sm_conf_destroy(stream);
		return 2;
	}

	if ((err = sm_conf_scan(stream, mts_def, 0, &mts)) != 0) {
		fprintf(stderr, "(while scanning) %s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));
		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			fprintf(stderr, "%s\n", e);
		sm_conf_destroy(stream);
		return 3;
	}
/*
	print_mts(&mts);
*/
	(void) sm_conf_prt_conf(mts_def, &mts, smioout);
/*
	(void) sm_conf_prt_dflt(mts_def, 0, smioout);
*/
	sm_io_flush(smioout);

	return 0;
}

int
main(int ac, char **av)
{
	int ai, c, ret, done;

	done = 0;
	while ((c = getopt(ac, av, "f:N:o:v:")) != -1) {
		switch (c) {
		  case 'f':
			ret = process(optarg, NULL);
			if (ret != 0)
				return ret;
			done = 1;
			break;
		}
	}
	ac -= optind;
	av += optind;
	if (done) {
		print_mts(&mts);
		return 0;
	}

	if (ac == 0)
		return process("*stdin*", stdin);

	for (ai = 0; ai < ac; ai++) {
		int ret = process(av[ai], NULL);
		if (ret != 0)
			return ret;
	}
	return 0;
}
