#!/bin/sh
# $Id: t-dkim-conf-2.sh,v 1.7 2013/12/13 02:31:41 ca Exp $
# Copyright (c) 2011 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test dkim signing: using a configuration file, add headers
# ----------------------------------------

test -s stop && exit 1              

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../libdkim
fi
. ${SD}/common.sh

# ----------------------------------------
# add a header
TEST=1
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="PaZQ9XXmPOaKzKqr3RdmKK9SCIi4JxIvz59LFcUGWPthH/w3MxSiX7GuXWKsB2hWwuiFe62Bg1c1vqK6ExyObRY8pfa949GOYxSpDW9uH6o11M1XHCfm8auEHNl+hnFid6BAbM+jTBR+S4ZVVxMAQ3oc3F5lQAaI21sPXpa9YQ0="

cat >$BODY <<EOF
Received: from client.example.com (tls.example.com [10.246.26.40])
	by server.example.com (MTA-5.1.4/MTA-5.1.0) with ESMTP id i1L137WP002508
	(version=TLSv1/SSLv3 cipher=EDH-RSA-DES-CBC3-SHA bits=168 verify=OK);
	Fri, 20 Feb 2010 17:03:08 -0800 (PST)
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM
Received: from client.example.net (client.example.net [10.246.26.40])
	by client.example.com (MTA-5.1.0/MTA-5.1.0) with ESMTP id i1L137WP002508
	(version=TLSv1/SSLv3 cipher=EDH-RSA-DES-CBC3-SHA bits=168 verify=OK);
	Fri, 20 Feb 2010 17:03:07 -0800 (PST)

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
skip_header = { "Received:" };
always_sign_headers = { "Cc" };
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
}
EOF
$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# header already exists
TEST=2
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="hU1eiDdhQCwLZh1jXoUNF9Sg++8hJfXb4823xLTxNK2y6nMc6M6/iRudoNbF8/ijn/by/PnVE94Jphmzbr9OCfwh0zy+L9wd8ajH7ge7zPjQgUvuiLIzf33GtHYSrSGG4ry+LkUJNcKvjxI7UDSRlnodX6X3V6dWpto6l06yNeU"

cat >$BODY <<EOF
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
always_sign_headers = { "Subject:" };
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
}
EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi




# ----------------------------------------
if test ${ERRS} -eq 0
then
  rm -f ${BODY} ${OUT} ${CONF}
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
