/*
 * Copyright (c) 2010 Claus Assmann
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the license/LICENSE.3C file which can be found at the
 * top level of this source code distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: compoptions.c,v 1.28 2014/02/16 01:00:43 ca Exp $")
#include "sm/common.h"
#include "sm/io.h"
#include "sm/misc.h"

/*
**  COMPOPTIONS -- show compile time options
**
**	Parameters:
**		fp -- output file
**
**	Returns:
**		none
*/

/*
 *  Too many options printed here? Filter somehow?
 *  one trick would be to use && SOME_FLAG and include the file in each
 *  caller instead putting it into a library.
 *  another option is to pass flags to this function and use multiple
 *  printf statements; grouped by flags.
 */

void
compoptions(sm_file_T *fp)
{
	sm_io_fprintf(fp,
		"compile time options:\n"
#if AQ_OOCC
		"AQ_OOCC\n"
#endif
#if FFR_AVLSLOTS
		"FFR_AVLSLOTS\n"
#endif
#if MTA_USE_TLS && FFR_CERT_PINNING
		"FFR_CERT_PINNING\n"
#endif
#if FFR_CLIENT_BIND
		"FFR_CLIENT_BIND\n"
#endif

/* qmgr */
#if MTA_DELAYED_DSN
		"MTA_DELAYED_DSN\n"
#endif

#if FFR_FBMX
		"FFR_FBMX\n"
#endif

#if MTA_NETINET6
		"MTA_NETINET6\n"
#endif
#if MTA_LIBDNS_TEST
		"MTA_LIBDNS_TEST\n"
#endif
#if MTA_USE_DKIM
		"MTA_USE_DKIM\n"
#endif
#if MTA_USE_TLS
		"MTA_USE_TLS\n"
# if MTA_HAVE_ECDSA
		"MTA_HAVE_ECDSA\n"
# endif
#endif
#if MTA_USE_TLSFIPS
		"MTA_USE_TLSFIPS\n"
#endif
#if MTA_TLS_DEBUG
		"MTA_TLS_DEBUG\n"
#endif
#if FFR_TLS_RETRY_IN_SMTPC
		"FFR_TLS_RETRY_IN_SMTPC\n"
#endif

#if MTA_USE_SASL
		"MTA_USE_SASL\n"
#endif
#if MTA_TEST_SASL
		"MTA_TEST_SASL\n"
#endif
#if MTA_USE_PMILTER
		"MTA_USE_PMILTER\n"
#endif
#if MTA_USE_RSAD
		"MTA_USE_RSAD\n"
#endif

#if OCC_RATE
		"OCC_RATE\n"
#endif

#if QMGR_DEBUG
		"QMGR_DEBUG\n"
#endif
#if QMGR_STATS
		"QMGR_STATS\n"
#endif
#if QMGR_TEST
		"QMGR_TEST\n"
#endif
#if QMGR_TEST_NO_AQUPDATE
		"QMGR_TEST_NO_AQUPDATE\n"
#endif
#if QM_TIMING
		"QM_TIMING\n"
#endif

#if RCBCOMM_DEBUG
		"RCBCOMM_DEBUG\n"
#endif

#if SC_DEBUG
		"SC_DEBUG\n"
#endif
#if SC_PIPELINING
		"SC_PIPELINING\n"
#endif
#if SC_SESS_FEATS
		"SC_SESS_FEATS\n"
#endif
#if SC_STATS
		"SC_STATS\n"
#endif
#if SC_TEST
		"SC_TEST\n"
#endif

#if SM_HEAP_CHECK
		"SM_HEAP_CHECK\n"
#endif
#if SMAR_DEBUG
		"SMAR_DEBUG\n"
#endif

#if SMAR_RVRS_ALL
		"SMAR_RVRS_ALL\n"
#endif
#if SMAR_TEST
		"SMAR_TEST\n"
#endif
#if SMAR_USE_DNS
		"SMAR_USE_DNS\n"
#endif

#if SS_CHECK_LINE_LEN
		/* untested feature */
		"SS_CHECK_LINE_LEN\n"
#endif

#if SM_TEST_HDRMOD
		"SM_TEST_HDRMOD\n"
#endif

#if SS_COND
		"SS_COND\n"
#endif
#if SS_EHLO_ACCESS_CHK
		"SS_EHLO_ACCESS_CHK\n"
#endif
#if SSQ_DEBUG
		"SSQ_DEBUG\n"
#endif
#if SS_TEST
		"SS_TEST\n"
#endif
		);
	sm_io_flush(fp);
}
