/*
 * Copyright (c) 2003-2006 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 * $Id: log.h,v 1.8 2014/03/14 17:43:53 ca Exp $
 */

#ifndef LOG_H
#define LOG_H 1

#include "sm/generic.h"
#include "sm/error.h"
#include "sm/time.h"
#include "sm/magic.h"
#include "sm/log.h"
#include "sm/syslog.h"
#include "sm/str.h"
#if MTA_USE_PTHREADS
#include "sm/pthread.h"
#endif

#define SM_LOG_MAGIC SM_MAGIC('S', 'L', 'O', 'G')

#define LOG_ENTRIES_MAX	10000

/* can syslog() deal with these lengths? */
#ifdef MTA_LOG_LEN
# define LOG_LEN MTA_LOG_LEN
#else
# define LOG_LEN 1022
#endif
#ifdef MTA_LOG_LEN_MAX
# define LOG_LEN_MAX MTA_LOG_LEN_MAX
#else
# define LOG_LEN_MAX 8192
#endif

#if LOG_LEN > LOG_LEN_MAX
 ERROR _LOG_LEN > _LOG_LEN_MAX: LOG_LEN > LOG_LEN_MAX
#endif

/* should be dynamically allocated later on */
#define SM_LOG_CATS_MAX	32
#define SM_LOG_MODS_MAX	32

struct sm_log_ctx_S
{
	sm_magic_T		 sm_magic;
	sm_logconfig_P	 lctx_cfg;
	uint			 lctx_dbg_level;
	uint32_t		 lctx_dont;
	uint32_t		 lctx_flags;
	int				 lctx_fd;
	sm_file_T		*lctx_fp;
	char			*lctx_filename;
	sm_str_P		 lctx_str;
	sm_logcategory_P lctx_cats[SM_LOG_CATS_MAX];
	uint			 lctx_cat_n;
	sm_logmodule_P	 lctx_mods[SM_LOG_MODS_MAX];
	uint			 lctx_mod_n;
#if SM_LOG_ROTATE
	/* doesn't work that simple: needs filename */
	time_T			 lctx_rotated;
	uint			 lctx_entries;
#endif
#if MTA_USE_PTHREADS
	pthread_mutex_t	 lctx_mutex;
#endif
};

#define SM_IS_LOGCTX(lctx) SM_REQUIRE_ISA((lctx), SM_LOG_MAGIC)

/* move to magic.h? */
#define SM_LOGCFG_MAGIC SM_MAGIC('S', 'L', 'C', 'F')

struct sm_logconfig_S
{
	sm_magic_T	 sm_magic;
};
#define SM_IS_LOGCFG(lcfg) SM_REQUIRE_ISA((lcfg), SM_LOGCFG_MAGIC)

#if MTA_USE_PTHREADS
# define LOCK_LOG do {	\
	int r = pthread_mutex_lock(&lctx->lctx_mutex);	\
	SM_LOCK_OK(r);	\
	} while (0)

# define UNLOCK_LOG do {	\
	int r = pthread_mutex_unlock(&lctx->lctx_mutex);	\
	SM_UNLOCK_OK(r);	\
	} while (0)

#else /* MTA_USE_PTHREADS */
# define LOCK_LOG
# define UNLOCK_LOG
#endif /* MTA_USE_PTHREADS */

#define LOG_MICRO(lctx) (((lctx)->lctx_flags & LOG_FL_MICRO) != 0)

#endif /* LOG_H */
