/*
 * Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: mapconfopen.c,v 1.13 2009/07/03 04:47:35 ca Exp $")

#include "sm/error.h"
#include "sm/string.h"
#include "sm/net.h"
#include "sm/memops.h"
#include "sm/maps.h"
#include "sm/mapc.h"
#include "sm/map.h"
#include "map.h"
#include "sm/sm-conf.h"
#include "sm/mapcnf.h"
#include "sm/mapconf.h"
#include "sm/confsetpath.h"

/*
**  SM_MAPCONFOPEN -- open all maps that are in a map declaration
**
**	Parameters:
**		mapdef -- map configuration
**		maps -- map system context
**		confdir -- directory of configuration file (for relative paths)
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
sm_mapconfopen(map_decl_P mapdef, sm_maps_P maps, const char *confdir)
{
	uint n;
	sm_ret_T ret;
	mapspec_P md_maps;

	SM_REQUIRE(mapdef != NULL);
	SM_REQUIRE(maps != NULL);
	for (n = 0; n < mapdef->mapdecl_n; n++) {
		md_maps = mapdef->mapdecl_maps + n;
		if (NULL == md_maps)
			continue;
		ret = sm_mapcnfopen(mapdef, maps, confdir, md_maps);
		if (sm_is_err(ret))
			goto error;
	}
	return SM_SUCCESS;

  error:
	return ret;
}
